package fr.android.pointspermis.settings;

import android.annotation.TargetApi;
import android.content.Context;
import android.content.SharedPreferences;
import android.content.res.Configuration;
import android.media.Ringtone;
import android.media.RingtoneManager;
import android.net.Uri;
import android.os.Build;
import android.os.Bundle;
import android.preference.ListPreference;
import android.preference.Preference;
import android.preference.PreferenceActivity;
import android.preference.PreferenceFragment;
import android.preference.PreferenceManager;
import android.support.v4.app.NavUtils;
import android.text.TextUtils;
import android.view.MenuItem;

import java.util.List;

import fr.android.pointspermis.R;

/**
 * A {@link android.preference.PreferenceActivity} that presents a set of application settings. On
 * handset devices, settings are presented as a single list. On tablets,
 * settings are split by category, with category headers shown to the left of
 * the list of settings.
 * <p/>
 * See <a href="http://developer.android.com/design/patterns/settings.html">
 * Android Design: Settings</a> for design guidelines and the <a
 * href="http://developer.android.com/guide/topics/ui/settings.html">Settings
 * API Guide</a> for more information on developing a Settings UI.
 */
public class ConfigActivity extends PreferenceActivity {

    public final static String DOSSIER_KEY = "dossier";
    public final static String DOSSIER1_KEY = "dossier1";
    public final static String DOSSIER2_KEY = "dossier2";
    public final static String DOSSIER3_KEY = "dossier3";
    public final static String CODE_KEY = "code";

    /**
     * Determines whether to always show the simplified settings UI, where
     * settings are presented in a single list. When false, settings are shown
     * as a master/detail two-pane view on tablets. When true, a single pane is
     * shown on tablets.
     */
    private static final boolean ALWAYS_SIMPLE_PREFS = false;

    // Shared Prefs
    private SharedPreferences prefs;
    private static SharedPreferences.Editor editor;


    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        // Shared prefs
        prefs = getSharedPreferences("fr.android.pointspermis", MODE_PRIVATE);
        editor = prefs.edit();
    }

    @Override
    protected void onPostCreate(Bundle savedInstanceState) {
        super.onPostCreate(savedInstanceState);

        // Add the 'go back' navigation button (top left)
        getActionBar().setDisplayHomeAsUpEnabled(true);

        // Show the settings
        setupSimplePreferencesScreen();
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
            // Respond to the action bar's Up/Home button
            case android.R.id.home:
                NavUtils.navigateUpFromSameTask(this);
                return true;
        }
        return super.onOptionsItemSelected(item);
    }

    /**
     * Shows the simplified settings UI if the device configuration if the
     * device configuration dictates that a simplified, single-pane UI should be
     * shown.
     */
    private void setupSimplePreferencesScreen() {

        if (!isSimplePreferences(this)) {
            return;
        }

        // In the simplified UI, fragments are not used at all and we instead
        // use the older PreferenceActivity APIs.

        // Add 'connection' preferences.
        addPreferencesFromResource(R.xml.pref_connection);

        // Bind the summaries of EditText/List/Dialog/Ringtone preferences to
        // their values. When their values change, their summaries are updated
        // to reflect the new value, per the Android Design guidelines.
        manageValueAndSummary(findPreference(DOSSIER_KEY));
        manageValueAndSummary(findPreference(CODE_KEY));
    }

    /**
     * A preference value change listener that updates the preference's summary
     * to reflect its new value.
     */
    private static Preference.OnPreferenceChangeListener changeListener = new Preference.OnPreferenceChangeListener() {
        @Override
        public boolean onPreferenceChange(Preference preference, Object value) {

            if (preference.hasKey()) {

                String key = preference.getKey();

                if (key.equals(DOSSIER_KEY)) {

                    /* No need to save values or update the summary as this is already done in
                    *  custom dialogPreference, but anyway we still do it ..
                    */

                    // Update the summary
                    ((DossierDialogPreference) preference).updateSummary();

                    // Get and save the dossier number
                    String dossier1 = ((DossierDialogPreference) preference).getDossier1();
                    String dossier2 = ((DossierDialogPreference) preference).getDossier2();
                    String dossier3 = ((DossierDialogPreference) preference).getDossier3();
                    editor.putString(DOSSIER1_KEY, dossier1);
                    editor.putString(DOSSIER2_KEY, dossier2);
                    editor.putString(DOSSIER3_KEY, dossier3);
                    editor.putString(DOSSIER_KEY, dossier1 + " " + dossier2 + " " + dossier3);
                }
                if (key.equals(CODE_KEY)) {

                    /* Do not change summary: code is private ! */

                    // Save the value
                    editor.putString(key, value.toString());
                }

                editor.commit();
            }

            return true;
        }
    };

    /**
     * Binds a preference's summary to its value. More specifically, when the
     * preference's value is changed, its summary (line of text below the
     * preference title) is updated to reflect the value. The summary is also
     * immediately updated upon calling this method. The exact display format is
     * dependent on the type of preference.
     *
     * @see #changeListener
     */
    private static void manageValueAndSummary(Preference preference) {

        // Set the listener to watch for value changes.
        preference.setOnPreferenceChangeListener(changeListener);

        String key = preference.getKey();

        // Trigger the listener immediately with the preference's current value to
        // update summary on element we want

        // /!\ not for dossier as it is self-managed neither code: not secure /!\

        /* Text parameters
        if (key.equals(RINGTONE_KEY) || key.equals(FREQUENCY_KEY)) {

            // Safe list default changes
            if (!PreferenceManager
                    .getDefaultSharedPreferences(preference.getContext())
                    .getString(preference.getKey(), "").equals("") ) {

                changeListener.onPreferenceChange(preference,
                        PreferenceManager
                                .getDefaultSharedPreferences(preference.getContext())
                                .getString(preference.getKey(), "")
                );
            }
        }*/
    }

    /**
     * This fragment shows general preferences only. It is used when the
     * activity is showing a two-pane settings UI.
     */
    @TargetApi(Build.VERSION_CODES.HONEYCOMB)
    public static class ConnectionPreferenceFragment extends PreferenceFragment {
        @Override
        public void onCreate(Bundle savedInstanceState) {
            super.onCreate(savedInstanceState);
            addPreferencesFromResource(R.xml.pref_connection);

            // Bind the summaries of EditText/List/Dialog/Ringtone preferences
            // to their values. When their values change, their summaries are
            // updated to reflect the new value, per the Android Design
            // guidelines.
            manageValueAndSummary(findPreference(DOSSIER_KEY));
            manageValueAndSummary(findPreference(CODE_KEY));
        }
    }

    /**
     * This fragment shows notification preferences only. It is used when the
     * activity is showing a two-pane settings UI.
     */
    @TargetApi(Build.VERSION_CODES.HONEYCOMB)
    public static class NotificationPreferenceFragment extends PreferenceFragment {
        @Override
        public void onCreate(Bundle savedInstanceState) {
            super.onCreate(savedInstanceState);
        }
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean onIsMultiPane() {
        return isXLargeTablet(this) && !isSimplePreferences(this);
    }

    /**
     * Helper method to determine if the device has an extra-large screen. For
     * example, 10" tablets are extra-large.
     */
    private static boolean isXLargeTablet(Context context) {
        return (context.getResources().getConfiguration().screenLayout
                & Configuration.SCREENLAYOUT_SIZE_MASK) >= Configuration.SCREENLAYOUT_SIZE_XLARGE;
    }

    /**
     * Determines whether the simplified settings UI should be shown. This is
     * true if this is forced via {@link #ALWAYS_SIMPLE_PREFS}, or the device
     * doesn't have newer APIs like {@link android.preference.PreferenceFragment}, or the device
     * doesn't have an extra-large screen. In these cases, a single-pane
     * "simplified" settings UI should be shown.
     */
    private static boolean isSimplePreferences(Context context) {
        return ALWAYS_SIMPLE_PREFS
                || Build.VERSION.SDK_INT < Build.VERSION_CODES.HONEYCOMB
                || !isXLargeTablet(context);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    @TargetApi(Build.VERSION_CODES.HONEYCOMB)
    public void onBuildHeaders(List<Header> target) {
        if (!isSimplePreferences(this)) {
            loadHeadersFromResource(R.xml.pref_headers, target);
        }
    }
}
